
------------------------------
-- Containers
------------------------------

tiles = {}

------------------------------
-- Misc Variables
------------------------------

local screen =  {love.window.getWidth(), love.window.getHeight()}
local W_half, H_half = love.window.getWidth()/2, love.window.getHeight()/2

light_position = {1, 1, 0}
object_position = {}
tile_size = 128

------------------------------




---------------------------------------------------
-- Tile class
---------------------------------------------------
Tile = {}
Tile.__index = Tile

-- Class methods
function Tile.init(subtype, x, y, extra)
	local self = {}

	--Basic Attributes
	self.subtype = subtype
	self.x, self.y, self.z = x, y, 0
	self.size = {tile_size, tile_size}
	self.texture = tile_tex

	-- Shadow Attributes
	self.shadowTexture = tile_tex

	self.Coords = {
								{0, 0, 0, 0},
								{self.size[1], 0, 1, 0},
								{self.size[1], self.size[2], 1, 1},
								{0, self.size[2], 0, 1}
							}

	self.shadowCoords = {
								{0, 0, 0, 0},
								{1, 0, 1, 0},
								{1, 1, 1, 1},
								{0, 1, 0, 1}
							}
	self.shadowGeometry =  love.graphics.newMesh(self.shadowCoords, self.shadowTexture)


	----------------
	setmetatable(self, Tile)
	return self
end

function Tile.initPoly(subtype, x, y, _Coords)
	local self = {}

	--Basic Attributes
	self.subtype = subtype
	self.x, self.y, self.z = x, y, 0
	self.size = {tile_size, tile_size}
	self.texture = tile_tex

	-- Shadow Attributes
	self.shadowTexture = tile_tex

	self.Coords = _Coords

	self.shadowCoords = {	{0, 0, 0, 0},
							{1, 0, 1, 0},
							{1, 1, 1, 1},
							{0, 1, 0, 1}  }
	self.shadowGeometry =  love.graphics.newMesh(self.shadowCoords, self.shadowTexture)
	
	self.Geometrycanvas = love.graphics.newCanvas( )
	
	love.graphics.setCanvas(self.Geometrycanvas)
	self.Geometrycanvas:clear()
	love.graphics.setBlendMode('alpha')
	love.graphics.setColor(255, 0, 0, 128)
	
	local coordsPoly ={}
	for i =1, #self.Coords  do
		coordsPoly[#coordsPoly+1]=self.Coords[i][1]
		coordsPoly[#coordsPoly+1]=self.Coords[i][2]
	end
	
	triangles = love.math.triangulate( coordsPoly )
	for i =1, #triangles  do
		love.graphics.polygon('fill', triangles[i])
	end
	--love.graphics.polygon('fill', coordsPoly)
	love.graphics.setCanvas()
	

	----------------
	setmetatable(self, Tile)
	return self
end


function Tile.render(self)

	--DRAW SHADOWS

	-- Set color to black
	love.graphics.setColor(0, 0, 0, 255)
	love.graphics.setBlendMode("alpha")

--if false then
	-- Set Distortion shader
	love.graphics.setShader(effect)

		-- Send Shader Info
	effect:send('light_position', {light_position[1], light_position[2], light_position[3]})
	effect:send('object_position', {self.x, self.y, 0})
	
	for i =1, #self.Coords -1 do
		effect:send('fixed_edge', {self.Coords[i][1], self.Coords[i][2], self.Coords[i+1][1], self.Coords[i+1][2]})
		love.graphics.draw(self.shadowGeometry, self.x, self.y, 0, 1, 1)
	end
	effect:send('fixed_edge', {self.Coords[#self.shadowCoords][1], self.Coords[#self.shadowCoords][2], self.Coords[1][1], self.Coords[1][2]})
	love.graphics.draw(self.shadowGeometry, self.x, self.y, 0, 1, 1)
--end
	


	--TOP SHADOW
	--effect:send('free_edge', {0, tile_size, tile_size, tile_size})
--	effect:send('fixed_edge', {0, 0, tile_size, 0})
--	love.graphics.draw(self.shadowGeometry, self.x, self.y, 0, 1, 1)

	--RIGHT SHADOW
	--effect:send('free_edge', {0, 0, 0, tile_size})
--	effect:send('fixed_edge', {tile_size, 0, tile_size, tile_size})
--	love.graphics.draw(self.shadowGeometry, self.x, self.y, 0, 1, 1)

	--BOTTOM SHADOW
	--effect:send('free_edge', {0, 0, tile_size, 0})
--	effect:send('fixed_edge', {0, tile_size, tile_size, tile_size})
--	love.graphics.draw(self.shadowGeometry, self.x, self.y, 0, 1, 1)

	--LEFT SHADOW
	--effect:send('free_edge', {tile_size, 0, tile_size, tile_size})
--	effect:send('fixed_edge', {0, 0, 0, tile_size})
--	love.graphics.draw(self.shadowGeometry, self.x, self.y, 0, 1, 1)


	love.graphics.setShader()


	-- DRAW OBJECT GRAPHICS
	love.graphics.setColor(255, 0, 0, 255)
	if self.Geometrycanvas then
		love.graphics.draw(self.Geometrycanvas, self.x, self.y)
	else
		love.graphics.draw(self.texture, self.x, self.y)
	end
end
---------------------------------------------------



function love.load()

	--LOAD SHADERS
	effect = love.graphics.newShader('effect.glsl')

	-- Load images
	tile_tex = love.graphics.newImage("tile.png")
	ball_tex = love.graphics.newImage("ball_64.png")

	-- Create Tiles
	tiles[#tiles+1] = Tile.init('basic', 384, 128)

	tiles[#tiles+1] = Tile.init('basic', 10, 10)
	
	tiles[#tiles+1] = Tile.init('basic', 200, 128)
	
	tiles[#tiles+1] = Tile.initPoly('basic', 400, 400,
							{
								{0, 0, 0, 0},
								{100, 0, 1, 0},
								{100, 100, 1, 1},
								{50, 150, 1, 1},
								{0, 100, 0, 1},
								{0, 0, 0, 0}
							})
	
	tiles[#tiles+1] = Tile.initPoly('basic', 600, 200,
							{
								{0, 0, 0, 0},
								{tile_size/2, 0, 1, 0},
								{tile_size, tile_size/2, 1, 1},
								{tile_size, tile_size, 1, 1},
								{tile_size/2, tile_size, 0, 1},
								{tile_size/2, (tile_size/2)*3, 0, 0},
								{(tile_size/2)*3, (tile_size/2)*3, 0, 0},
								{(tile_size/2)*3, 0, 0, 0},
								{tile_size*2, tile_size/2, 0, 0},
								{tile_size*2, tile_size*2, 0, 0},
								{0, tile_size*2, 0, 0},
								{0, 0, 0, 0},
							})	
	
	
end



-- QUIT FUNCTION
function love.keyreleased(key)

   if key == "escape" then
		love.event.push("quit")
	end

end


-- UPDATE STUFF
function love.update(dt)

	light_position[1] = love.mouse.getX()
	light_position[2] = love.mouse.getY()

	-- move tiles
	for i =1, #tiles do

		if love.keyboard.isDown("w") then
			tiles[i].y = tiles[i].y - 200 *dt

		elseif love.keyboard.isDown("s") then
			tiles[i].y = tiles[i].y + 200 *dt

		end


		if love.keyboard.isDown("a") then
			tiles[i].x = tiles[i].x - 200 *dt

		elseif love.keyboard.isDown("d") then
			tiles[i].x = tiles[i].x + 200 *dt

		end

	end

	object_position = {tiles[1].x/screen[1], tiles[1].y/screen[2], 0}

end



-- DRAW STUFF
function love.draw()

	love.graphics.setBackgroundColor(150, 150, 150)
	love.graphics.clear( )

	--Draw Tiles
	for tile = 1, #tiles do

		local object = tiles[tile]

		object:render()

	end


	-- Draw ball at light position
	love.graphics.setColor(0, 255, 0, 255)
	love.graphics.draw(ball_tex, light_position[1], light_position[2], 0, 0.25, 0.25, 32, 32)

	------------------------
	--Debug Info
	------------------------
	love.graphics.setColor(255, 255, 100, 255)
	love.graphics.setBlendMode("alpha")

	love.graphics.print("tile: ".."x:"..tiles[1].x, 300, 10)
	love.graphics.print("tile: ".."y:"..tiles[1].y, 300, 20)

	love.graphics.print("light: ".."x:"..light_position[1], 600, 10)
	love.graphics.print("light: ".."y:"..light_position[2], 600, 20)

	love.graphics.print("Current FPS: "..tostring(love.timer.getFPS( )), 10, 10)

end

